% Main File for Structural Model
% for "Mortgage Default during the U.S. Mortgage Crisis"
% by Thomas Schelkle
% Uses: distancefunc, solvepolicy, simulmodel, params
%--------------------------------------------------------
%--------------------------------------------------------
% BEFORE RUNNING THE CODE: Create the resultspath and graphspath folders on your disk!
%--------------------------------------------------------
%--------------------------------------------------------

%clear all;
close all;

%--------------------------------------------------------
%--------------------------------------------------------
% Options and Parameters

% Paths 
comp = 1; % computation number can be used to save results and graphs in specific folder (first create subfolders in results and graphs folder)
datapath = 'data\';
resultspath = ['results\comp',num2str(comp),'\'];
graphspath =['graphs\comp',num2str(comp),'\'];

% Data Options 
defdatanumb = 60; % 60=60+delinqu,120=120+delinqu, 180=FCPresale, 200="Foreclosure completed" definition
backdate = 1; % backdate default data: 0=not, 1= by one month, 3=by three months, 6=by six months
LTVnumb = 10; % 10=95-104% LTV bin

% Estimation Options
opt_estalg = 1; %0=no estimation (just solve for a given parameter), 1=simple grid search
opt_solvesimul = [true;true]; %(1): true=solve for policy functions, (2): true=simulate model (false=use stored results)
opt_rentersolve = true; % true=solve renter subproblem (false=use stored results) 
theta = 0.28; % value of theta used for opt_estalg = 0 (no estimation)
thetavec = [0.26:0.01:0.3]'; % grid of theta values for opt_estalg = 1 (estimation by grid search) 


%--------------------------------------------------------
%--------------------------------------------------------
% Load Mortgage Data

load([datapath,'FRMdata']);

if defdatanumb == 200, % "Foreclosure completed" default definition
    Data_defcum = FCcompleted_ac_cumul(:,:,LTVnumb+1);
elseif defdatanumb ==180, % Foreclosure Presale default definition
    Data_defcum = DefTerminationsFCPre_ac_cumul(:,:,LTVnumb+1);
elseif defdatanumb ==120, % 120+ days default definition
    Data_defcum = DefTerminations120_ac_cumul(:,:,LTVnumb+1);
else % 60+ days default definition
    Data_defcum = DefTerminations_ac_cumul(:,:,LTVnumb+1);
end
Data_defcum = Data_defcum(1+backdate:end,:); % backdate default
Data_defcum(isnan(Data_defcum)) = 0;

simultime = 2002:1/12:2010+5/12; % simulation period Jan 2002 to June 2010
simultime = simultime(1:end-backdate);


%--------------------------------------------------------
%--------------------------------------------------------
% Solve/Estimate Structural Model

ticstruct=tic;

if opt_estalg == 0, % solve for given parameter values (do not estimate)
    
    opt_solverentown = [opt_rentersolve;true]; %first arg: solve renter, second arg: solve owner
    distmeasure = distancefunc(theta,Data_defcum,simultime,LTVnumb,resultspath,datapath,opt_solvesimul,opt_solverentown);
    
elseif opt_estalg==1, % perform grid search
    
    % solve renter problem once for arbitrary theta (renter problem is independent of theta)
    if opt_solvesimul(1),
        opt_solverentown = [opt_rentersolve;false]; %first arg: solve renter, second arg: solve owner
        solvepolicy(theta,resultspath,datapath,opt_solverentown);
    end
    
    % solve owner problem for different thetas
    jobnumb = length(thetavec);
    disp(['Compute model for ',num2str(jobnumb),' different values of theta.']);
    distmeasure = zeros(jobnumb,1);
    opt_solverentown = [false;true]; %first arg: solve renter, second arg: solve owner
    parfor jobcounter = 1:jobnumb,
        thetajob = thetavec(jobcounter);
        distmeasure(jobcounter,:) = distancefunc(thetajob,Data_defcum,simultime,LTVnumb,resultspath,datapath,opt_solvesimul,opt_solverentown);
    end
    save([resultspath,'distancemeasure'],'thetavec','distmeasure');
    
    % choose theta such that distance to data is minimized
    [mindist,indmin] = min(distmeasure);
    theta = thetavec(indmin);
    disp(['Distance minimized for theta = ',num2str(theta),' with distance = ',num2str(mindist)])
    figure;plot(thetavec,distmeasure,'-o');xlabel('theta');ylabel('distance');
    
end

%--------------------------------------------------------
%--------------------------------------------------------
% Plot Graphs


% Plot Policy Functions (Figure 5 in Paper)
params;
load([resultspath,'owner_theta',num2str(1000*theta)]);
for t = 1:19:20, % plot policy for period 1 and 20 ...
    if gp_agg>1,
        g = round(gp_agg/2);
    else
        g = 1;
    end
    for e = 1:E, % ... and being employed/unemployed
        X1 = linspace(GX(1,t),GX(end,t),10*gp_x0);
        X2 = linspace(GPH(1,t),GPH(gp_h-3,t),10*gp_h);
        if opt_simulinterppolicy,
            I1 = interpn(GX(:,t),GPH(:,t),I(:,:,g,e,t),X1',X2,'nearest');
        else
            V3 = interpn(GX(:,t),GPH(:,t),squeeze(V(:,:,g,e,t)),X1',X2,interpmethsolve);
            XS = repmat(X1',1,length(X2)) + repmat(X2,length(X1),1) - M(t);
            V2 = interp1(GXr(:,t),squeeze(Vr(:,e,t)),XS,interpmethsolve);
            XD = repmat(X1',1,length(X2));
            V1 = interp1(GXr(:,t),squeeze(Vr(:,e,t)),XD,interpmethsolve);
            I1 = 3*ones(length(X1),length(X2));
            I1((V2>V3)&(repmat(X2,length(X1),1)>=M(t))) = 2;
            I1((V1>V3)&(repmat(X2,length(X1),1)<M(t))) = 1;
        end
        figure;
        [Co,ho] =contourf(X1,X2-M(t),I1',[1 2 3]);
        colormap([0 0 0;1 1 1;0.5 0.5 0.5]);
        caxis([1 3.2]);
        xla=xlabel('liquid wealth');
        yla=ylabel('house equity');
        text(0.9,-40,'Default','FontSize',18,'Color','White');
        text(0.9,40,'Sell','FontSize',18,'Color','Black');
        text(3,0,'Stay','FontSize',18,'Color','White');
        ax=gca;
        axis([GX(1,t) 5 -60 60]);
        set(ax,'FontSize',14,'Fontweight','bold');
        set(xla,'Fontsize',14,'Fontweight','bold');
        set(yla,'FontSize',14,'Fontweight','bold');
        if t==1,
            if e==1,
                print('-depsc', [graphspath,'repaypolicy_empl']);
            else
                print('-depsc', [graphspath,'repaypolicy_unempl']);
            end
        else
            if e==1,
                print('-depsc', [graphspath,'repaypolicy_empl20']);
            else
                print('-depsc', [graphspath,'repaypolicy_unempl20']);
            end
        end
    end
end


% Aggreate Simulated and Empirical Default Rates to Yearly Cohort Level
load([resultspath,'simul_theta',num2str(1000*theta)]);
Tsimul = 84; % number of origination dates used in simulation (from Jan2002 to Dec2008)
Lmax = length(simultime); % maximum length of simulation for an individual cohort 
Lind = Lmax:-1:Lmax-Tsimul+1; % simulation length for individual cohort
time02 = (1:Lmax)';
model = NaN(Lmax,7); 
data = NaN(Lmax,7);
ph_data = NaN(Lmax,7);
for ji=1:7,
    model(1:Lind(12*ji),ji) = mean(Idefcum(1:Lind(12*ji),12*(ji-1)+1:12*ji),2);
    data(1:Lind(12*ji),ji) = mean(Data_defcum(1:Lind(12*ji),12*(ji-1)+1:12*ji),2);
    ph_data(1:Lind(12*ji),ji) = mean(ph_agg(1:Lind(12*ji),12*(ji-1)+1:12*ji),2);
end


% Figure 6 in Paper: Compare Model to Data for 2002 Cohort
figure('Name','Comparing model to data for 2002 cohort');
set(gcf,'DefaultAxesColorOrder',[0.5 0.5 0.5;0 0 0]);
pl=plot(time02,100*data(:,1),'--',time02,100*model(:,1));
xla=xlabel('months since origination');
yla=ylabel('cumulative default rate in %');
le=legend(pl([2 1]),'model','data');
axis([0 100 0 100*0.08])
ax=gca;
set(pl,'Linewidth',3);
set(ax,'FontSize',14,'Fontweight','bold');
set(xla,'Fontsize',14,'Fontweight','bold');
set(yla,'FontSize',14,'Fontweight','bold');
print('-depsc', [graphspath,'default_model_data_2002']);


% Figure 7 in Paper: Compare Model to data for 2002-2008 cohorts
figure('Name','Comparing model to data for 2002-2008 cohorts');
colormat = [0 0 0;0.45 0.45 0.45;0.6 0.6 0.6;0 0 0;0.45 0.45 0.45;0.6 0.6 0.6;0 0 0];
set(gcf,'DefaultAxesColorOrder',colormat);
pl=plot(time02,100*data,'--',time02,100*model);
xla=xlabel('months since origination');
yla=ylabel('cumulative default rate in %');
for i = 1:7,
    timei = time02(not(isnan(ph_data(:,i))));
    datai = data(not(isnan(data(:,i))),i);
    modeli = model(not(isnan(model(:,i))),i);
    text(timei(end),100*max(datai(end),modeli(end)),num2str(2001+i),'FontSize',14,'Fontweight','bold','Color',colormat(i,:),'HorizontalAlignment','center','VerticalAlignment','bottom')
end
axis([0 100 0 100*(max([0.22,max(max(model+0.03))]))]);
ax=gca;
set(pl,'Linewidth',3);
set(ax,'FontSize',14,'Fontweight','bold');
set(xla,'Fontsize',14,'Fontweight','bold');
set(yla,'FontSize',14,'Fontweight','bold');
print('-depsc', [graphspath,'default_model_data_many']);


% Figure 1a in Paper: Default Rates in Data
figure;
colormat = [0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0];
set(gcf,'DefaultAxesColorOrder',colormat);
pl=plot(time02,100*data);
xla=xlabel('months since origination');
yla=ylabel('cumulative default rate in %');
for i = 1:7,
    timei = time02(not(isnan(ph_data(:,i))));
    datai = data(not(isnan(data(:,i))),i);
    text(timei(end),100*datai(end),num2str(2001+i),'FontSize',14,'Fontweight','bold','Color',colormat(i,:),'HorizontalAlignment','center','VerticalAlignment','bottom')
end
axis([0 100 0 100*0.22])
ax=gca;
set(pl,'Linewidth',3);
set(ax,'FontSize',14,'Fontweight','bold');
set(xla,'Fontsize',14,'Fontweight','bold');
set(yla,'FontSize',14,'Fontweight','bold');
print('-depsc', [graphspath,'default_data_many']);


% Figure 1b in Paper: House Price Paths 
figure;
colormat = [0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0];
set(gcf,'DefaultAxesColorOrder',colormat);
pl=plot(time02,ph_data);
xla=xlabel('months since origination');
yla=ylabel('mean real house price');
for i = 1:7,
    timei = time02(not(isnan(ph_data(:,i))));
    ph_datai = ph_data(not(isnan(ph_data(:,i))),i);
    text(timei(end),ph_datai(end),num2str(2001+i),'FontSize',14,'Fontweight','bold','Color',colormat(i,:),'HorizontalAlignment','center','VerticalAlignment','top')
end
axis([0 100 80 130]);
ax=gca;
set(pl,'Linewidth',3);
set(ax,'FontSize',14,'Fontweight','bold');
set(xla,'Fontsize',14,'Fontweight','bold');
set(yla,'FontSize',14,'Fontweight','bold');
print('-depsc', [graphspath,'ph_data_many']);

%--------------------------------------------------------
%--------------------------------------------------------
% Calculate GOODNESS OF FIT MEASURES

%Sum of squarred errors
t_fit_vec = 1:12; % fitted to 2002 cohort
SSE_in = sum(sum((Idefcum(:,t_fit_vec)-Data_defcum(:,t_fit_vec)).^2));
SSE_all = sum(sum((Idefcum-Data_defcum).^2));
%Sum of absolute errors
SAE_in = sum(sum(abs(Idefcum(:,t_fit_vec)-Data_defcum(:,t_fit_vec))));
SAE_all = sum(sum(abs(Idefcum-Data_defcum)));

SSE_out = SSE_all-SSE_in;
SAE_out = SAE_all-SAE_in;

disp('----------------------------------------------------------------------');
disp('Some Goodness of Fit Measures:')
disp(' ');
disp('Sum of Squarred Errors:   All   In-Sample   Out-of-Sample');
disp([num2str(SSE_all),'   ',num2str(SSE_in),'   ',num2str(SSE_out)]);
disp(' ');
disp('Sum of Absolute Errors:   All   In-Sample   Out-of-Sample');
disp([num2str(SAE_all),'   ',num2str(SAE_in),'   ',num2str(SAE_out)]);
disp('----------------------------------------------------------------------');


monthvec = 6:12:90;
tvec = 8:-1:2;
cohortvec = 2002:2008;
disp('----------------------------------------------------------------------');
disp('Cumulative Default Rates in Detail:');
disp(['Cohort &  & ',num2str(monthvec(1)),' & ',num2str(monthvec(2)),' & ',num2str(monthvec(3)),' & ',num2str(monthvec(4)),' & ',num2str(monthvec(5)),' & ',num2str(monthvec(6)),' & ',num2str(monthvec(7)),' & ',num2str(monthvec(8)),' \\']);
for i = 1:7,
    datavec = [data(monthvec(1:tvec(i)),i);NaN(8-tvec(i),1)]';
    modelvec = [model(monthvec(1:tvec(i)),i);NaN(8-tvec(i),1)]';
    disp([num2str(cohortvec(i)),' & Data & ',num2str(100*datavec(1),'% 10.1f'),' & ',num2str(100*datavec(2),'% 10.1f'),' & ',num2str(100*datavec(3),'% 10.1f'),' & ',num2str(100*datavec(4),'% 10.1f'),' & ',num2str(100*datavec(5),'% 10.1f'),' & ',num2str(100*datavec(6),'% 10.1f'),' & ',num2str(100*datavec(7),'% 10.1f'),' & ',num2str(100*datavec(8),'% 10.1f'),' \\']);
    disp([num2str(cohortvec(i)),' & Model & ',num2str(100*modelvec(1),'% 10.1f'),' & ',num2str(100*modelvec(2),'% 10.1f'),' & ',num2str(100*modelvec(3),'% 10.1f'),' & ',num2str(100*modelvec(4),'% 10.1f'),' & ',num2str(100*modelvec(5),'% 10.1f'),' & ',num2str(100*modelvec(6),'% 10.1f'),' & ',num2str(100*modelvec(7),'% 10.1f'),' & ',num2str(100*modelvec(8),'% 10.1f'),' \\']);
end

%--------------------------------------------------------
%--------------------------------------------------------
% Report Results of Policy Analysis (Table 4 in Paper)

if opt_bailout,
    meanAvg_NegEqu = zeros(1,7);
    meanAvg_Subsidy = zeros(1,7);
    for c=1:7,
        meanAvg_NegEqu(c) = mean(Avg_NegEqu(12*(c-1)+1:12*c));
        meanAvg_Subsidy(c) = mean(Avg_Subsidy(12*(c-1)+1:12*c));
    end
    disp('Results for Policy Analysis:');
    disp(['Cost of Bailout to Lenders:   ',num2str(-meanAvg_NegEqu,'% 10.1f')]);
    disp(['Cost of Subsidy to Borrowers:   ',num2str(meanAvg_Subsidy,'% 10.1f')]);
    disp(['Ratio Bailout/Subsidy:   ',num2str(-meanAvg_NegEqu./meanAvg_Subsidy,'% 10.1f')]);
end


%--------------------------------------------------------
%--------------------------------------------------------

toc(ticstruct);

    









