% Parameters and Options of Structural Model
% for "Mortgage Default during the U.S. Mortgage Crisis"
% by Thomas Schelkle
%--------------------------------------------------------
%--------------------------------------------------------

% MODEL PARAMETERS 

% Time frequency
frequ = 12; % frequency of the model: 1=yearly, 4=quarterly, 12=monthly 

% Preference parameters
betta = 0.9;
gama = 5;

% Mortgage Contract Details
T = 30; % length of contract in years
LTV = 0.982; % initial loan-to-value ratio (only for structural model)
DTI = 0.4; % initial loan(mortgage payment)-to-income ratio

% Initial conditions
PH_0 = 100; % initial price of house (normalized to 100)
M_0 = LTV*PH_0; % initial loan amaount 

% Prices
alpha = 0.04;% rent/price ratio
r = 0.014;% real interest rate on assets
delta = 1;% downsizing factor when renting (1=no downsizing)
rm = 0.064; % nominal mortgage rate (only for structural model)
pii = 0.024;% inflation rate

% Borrowing constraint
psii = 0; % amount that can be borrowed

% Stochastic House Prices
% Aggregate Component
hp_option = 1; %1=aggregate growth rate iid, 2=persistence in aggregate growth rate (option 2 not used in paper)
if hp_option == 1,% parameters for iid hp growth
    mu_rh_agg = [0.0078; 0.00195; 0.00065]; % yearly/quarterly/monthly
    s_rh_agg = [0.019;0.00953;0.0055]; % yearly/quarterly/monthly
elseif hp_option == 2, % parameters for persistent growth rate
    a_rh_agg = 0.0002992;
    b_rh_agg = 0.5217221;
    s_rh_agg = 0.00471;
end
% Individual Component
kappa = 0.001873517;
lambda = -4.50942E-06;

% Income Process
pieu = 0.018; % job destruction rate
piee = 1-pieu;
piue = 0.31; % job finding rate
piuu = 1-piue;

% policy parameters
tax = 0.16; % tax rate
rho = 0.62; % net replacement level of UI
opt_taxdeduct = true; %true=mortgage interest is tax deductable, false=not deductable

%--------------------------------------------------------
% ALGORITHM SETTINGS

interpmethsolve = 'spline';
interpmethsimul = 'linear';
disc_method = 1; % discretization for persistent hp component: 1=Tauchen, 2-4: Tauchen-Hussey with sigma(2), sigmaZ(3) or w*sigma+(1-w)*sigmaZ(4) (not used in paper)

% number and location of grid points 
numnodes_h = 5; % number of nodes used for discretization of the iid component of house price growth shocks (includes aggregate in iid case)
gp_x0 = 100; % grid points in cash-on-hand dimension
gp_h = 100;% grid points in house price dimension
gp_s0 = 1000; % grid points on saving grid
gp_agg = 5; % number of nodes used to discretize aggregate hp component in persistent case 
gp_x_factor = 10; % use more x grid points for renter by this factor
gp_s_factor = 5; % use more s grid points for renter by this factor
curvpara = 3; % curvature parameter for X and S grids

% grid limits
GPHupperbound = 2*PH_0; % upper bound of house price grid
GPHlowerbound = 0.3*PH_0; % lower bound of house price grid
GXup_factor = 6; % upper bound of X grid as multiple of maximum income

%--------------------------------------------------------
% SIMULATION SETTINGS

numi = 25000; % number of individuals used for monte carlo simulation
seed = 0; % common seed across labs
opt_simulinterppolicy =true; % true= interpolate policy function, false = interpolate value functions
opt_bufferA0 = true; %true = use desired buffer-stock as A0, false=use A0=0.0027 
opt_var_sf = false; %true = use variable separation and finding rates
tolconstr = 0.01; % tolerance for violations of borrowing constraint to be considered "severe"
opt_bailout = true; %true=do policy analysis, false = do not


%--------------------------------------------------------
%--------------------------------------------------------
% Adjust parameters to time frequency of the model
T = T*frequ;
J = T+1; % remaining life-time 
betta = betta^(1/frequ);
r = (1+r)^(1/frequ) - 1;
rm = (1+rm)^(1/frequ) - 1;
alpha = (1+alpha)^(1/frequ) -1;
pii = (1+pii)^(1/frequ) - 1;

var_rh_ind = kappa*(4/frequ) + lambda*(4/frequ)^2*(2*(1:J)-1); % variance of individual shock depends on time
s_rh_ind = sqrt(var_rh_ind);

if hp_option == 1,
    if frequ==1,
        mu_rh_agg = mu_rh_agg(1);
        s_rh_agg = s_rh_agg(1);
    elseif frequ==4
        mu_rh_agg = mu_rh_agg(2);
        s_rh_agg = s_rh_agg(2);
    elseif frequ==12
        mu_rh_agg = mu_rh_agg(3);
        s_rh_agg = s_rh_agg(3);
    end
    mu_rh = mu_rh_agg;
    s_rh = sqrt(s_rh_agg^2 + s_rh_ind.^2);
end

%--------------------------------------------------------
% Mortgage 

% Implied mortgage dynamics (nominal)
mn = ones(T,1)*M_0/sum((1+rm).^(-(1:T)));
Mn = zeros(T,1);
Mn(1) = M_0; 
for t=2:T,
    Mn(t) = (1+rm)*Mn(t-1) - mn(t-1);
end

% Deflate nominal variables Mn and mn to real variables M and m 
Pcons = (1+pii).^((1:T)'-1); % price of consumption goods
M = Mn./Pcons;
m = mn./Pcons;

% M and m are zero for t>T
M = [M;0]; 
m = [m;0];

% include tax deductability to arrive at effective mortgage payment
if opt_taxdeduct,
    m = m - tax*rm*M;
end 

%--------------------------------------------------------
% Rent
rent = delta*alpha*PH_0; 

%--------------------------------------------------------
% Income Process

Y_0 = mn(1)/DTI; % initial gross income
Yvec = (1-tax)*Y_0*[1;rho]; % net income vector
pimat = [piee pieu;piue piuu]; % transition matrix
E = length(Yvec); % number of income realizations
einit = 1; % index of initial income state in Yvec
Ymat = repmat(Yvec,1,J); % dimensions: E x J
Yinit = Ymat(einit,1); % initial net income
Ymax = max(Ymat); % maximum income
Ymin = min(Ymat); % minimum net income

%-------------------------------------------------------
% Borrowing Limit (artifical or natural)
PDVlowInc = zeros(J,1); % PDV of future income when being unemployed
PDVrent = zeros(J,1); % PDV of future rent
PDVm = zeros(J,1); % PDV of future mortgage payment
PDVminmr = zeros(J,1); % PDV of cheaper option
rentvec= rent * [ones(J-1,1);0];
for t = J-1:-1:0,
    if t>0,
        PDVlowInc(t) = Ymin(t+1)/(1+r) + PDVlowInc(t+1)/(1+r);
        PDVrent(t) = rentvec(t+1)/(1+r) + PDVrent(t+1)/(1+r);
        PDVm(t) = m(t+1)/(1+r) + PDVm(t+1)/(1+r);
        PDVminmr(t) = min(PDVrent(t),PDVm(t));
    else
        PDVlowInc0 = Ymin(t+1)/(1+r) + PDVlowInc(t+1)/(1+r);
        PDVrent0 = rentvec(t+1)/(1+r) + PDVrent(t+1)/(1+r);
        PDVm0 = m(t+1)/(1+r) + PDVm(t+1)/(1+r);
        PDVminmr0 = min(PDVrent0,PDVm0);
    end
end
BLrent = min(psii*ones(J,1),PDVlowInc-PDVrent); % effective borrowing limit of renter 
BL = min(psii*ones(J,1),PDVlowInc-PDVminmr); % effective borrowing limit of owner 
BLrent0 = min(psii,PDVlowInc0-PDVrent0); 
BL0 = min(psii,PDVlowInc0-PDVminmr0); 

%--------------------------------------------------------
% Discretize persistent house price process
if hp_option == 1,
    Z_agg = 0;
    PI_agg = 1;
    gp_agg = 1;
elseif hp_option == 2,
    rho_rh_agg = b_rh_agg;
    mu_rh_agg = a_rh_agg/(1-b_rh_agg);
    sigma_rh_agg = s_rh_agg;
    w_rh_agg = 0.5 + rho_rh_agg/4;
    sigmaZ_rh_agg = sigma_rh_agg/sqrt(1-rho_rh_agg^2);
    % discretize aggregate component use Tauchen's method
    if disc_method==1,
        [Z_agg,PI_agg] = tauchen(gp_agg,mu_rh_agg,rho_rh_agg,sigma_rh_agg,1.2*log(gp_agg));
    elseif disc_method==2,
        [Z_agg,PI_agg] = tauchenhussey(gp_agg,mu_rh_agg,rho_rh_agg,sigma_rh_agg,sigma_rh_agg);
    elseif disc_method==3,
        [Z_agg,PI_agg] = tauchenhussey(gp_agg,mu_rh_agg,rho_rh_agg,sigma_rh_agg,sigmaZ_rh_agg);
    elseif disc_method==4,
        [Z_agg,PI_agg] = tauchenhussey(gp_agg,mu_rh_agg,rho_rh_agg,sigma_rh_agg,w_rh_agg*sigma_rh_agg + (1-w_rh_agg)*sigmaZ_rh_agg);
    end
end

%--------------------------------------------------------
