% Read and Prepare Data
% for "Mortgage Default during the U.S. Mortgage Crisis"
% by Thomas Schelkle
%--------------------------------------------------------
%--------------------------------------------------------
% NOTE: Obtain the proprietary mortgage data directly from LPS and place it
% into the folder specified in LPSfolder. Download the data in Excel-format 
% using the specifications described in the comments below.
%--------------------------------------------------------
%--------------------------------------------------------

clear all;
close all;

datapath = 'data\'; % general data path
LPSfolder = 'LPSdata\'; % sub folder containing proprietary mortgage data (place this data here)  

%--------------------------------------------------------
% Read and prepare mortgage data

rownumb = 362;
colnumb = 138;
sheetnumb = 11;
maxage = 360;
age = (0:maxage)';
time = (1999:1/12:2010+5/12)';

% Create Variables of Interest
% row=age of loan, column=time period, sheets = original LTV range
% row 1: total of respective month (column) across all ages (rows)
% sheet 1: total across all LTVs (sheets)
LC = zeros(rownumb,colnumb,sheetnumb); % Loan Count
LTV = zeros(rownumb,colnumb,sheetnumb); % LTV
InterestRate = zeros(rownumb,colnumb,sheetnumb); % Interest Rate
FICO = zeros(rownumb,colnumb,sheetnumb); % FICO score
LoanAmount = zeros(rownumb,colnumb,sheetnumb); % Loan Amount
DQ60_MBA = zeros(rownumb,colnumb,sheetnumb); % fraction of loans 60-89 days delinquent (MBA)
DQ90_MBA = zeros(rownumb,colnumb,sheetnumb); % fraction of loans 90-119 days delinquent (MBA)
DQ120plus_MBA = zeros(rownumb,colnumb,sheetnumb); % fraction of loans 120 and more days delinquent (MBA)
Runoff = zeros(rownumb,colnumb,sheetnumb); % fraction of loans prepaying in t
FCPresale = zeros(rownumb,colnumb,sheetnumb); % fraction of loans in FC
FCcompleted = zeros(rownumb,colnumb,sheetnumb); % FC completed in t 

% Format of Excel files downloaded from LPS: 
% Variable as in name of file
% Rows: Age of Loan (Months): row 1: total, then 0-360
% Columns: Time Periods (Months): Jan 1999 - June 2010
% Sheets: 
% - Break 1: Original LTV Ratio (Ranges): Total,1-14,15-24,...,75-84,85-94,95 and more 
% - very first sheet contains an "Index" and very last the "Selections"
% Filter: Mortgage Type is First, and Interest Type is Fixed, and Initial Loan Term EQ 360, and Balloon Code is Not Balloon
disp('load default and contract characteristics data');
parfor page = 1:sheetnumb,
    disp(['sheet ',num2str(page),' of ',num2str(sheetnumb)]);
    LC(:,:,page) = xlsread([datapath,LPSfolder,'LoanCount.xls'],page+1);
    LTV(:,:,page) = xlsread([datapath,LPSfolder,'LTV.xls'],page+1);
    InterestRate(:,:,page) = xlsread([datapath,LPSfolder,'InterestRate.xls'],page+1);
    FICO(:,:,page) = xlsread([datapath,LPSfolder,'FICO.xls'],page+1);
    LoanAmount(:,:,page) = xlsread([datapath,LPSfolder,'LoanAmount.xls'],page+1);
    DQ60_MBA(:,:,page) = xlsread([datapath,LPSfolder,'60DQ_MBA.xls'],page+1);
    DQ90_MBA(:,:,page) = xlsread([datapath,LPSfolder,'90DQ_MBA.xls'],page+1);
    DQ120plus_MBA(:,:,page) = xlsread([datapath,LPSfolder,'120DQ+_MBA.xls'],page+1);
    Runoff(:,:,page) = xlsread([datapath,LPSfolder,'Runoff.xls'],page+1);
    FCPresale(:,:,page) = xlsread([datapath,LPSfolder,'FCPresale.xls'],page+1);
end

% Format of RunoffArray Excel files downloaded from LPS:
% Variable: Loan Count
% Rows: Age of Loan (Months): row 1: total, then 0-360
% Columns: Runoff Loan Array (All values)
% Sheets: 
% - Break 1: Original LTV Ratio (Ranges): Total,1-14,15-24,...,75-84,85-94,95 and more 
% - Break 2: Time Periods (Months): RunoffArray7: Jan1999-Dec1999, RunoffArray6: Jan 2000-Sept2001, ..., RunoffArray1: Oct2008-Jun2010
% - very first sheet contains an "Index" and very last the "Selections"
% Filter: Mortgage Type is First, and Interest Type is Fixed, and Initial Loan Term EQ 360, and Balloon Code is Not Balloon
numfile = 7;
disp('load FC completed data');
parfor ltv = 1:sheetnumb, % loop over LTV ranges
    FCcompleted_ltv = zeros(rownumb,colnumb);
    for f = 1:numfile, % loop over files
        disp([' ltv: ',num2str(ltv),' file: ',num2str(f)]);
        if f==7, % file 7 only contains 12 months
            tt = 12;
        else
            tt = 21;
        end
        for t = 1:tt,
            page = (ltv-1)*tt + t;
            Runtab = xlsread([datapath,LPSfolder,'RunoffArray',num2str(f),'.xls'],page+1);
            if f==7, % file 7 only contains 12 months
                timeperiod = t;
            else
                timeperiod = colnumb - f*tt + t;
            end
            FCcompleted_ltv(:,timeperiod) = Runtab(:,4);
        end
    end
    FCcompleted(:,:,ltv) = FCcompleted_ltv;
end

% create data over the course of the loan for each cohort
LC_ac = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
DQ60_MBA_ac = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
DQ90_MBA_ac = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
DQ120plus_MBA_ac = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
Runoff_ac = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
FCPresale_ac = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
FCcompleted_ac = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
for page = 1:sheetnumb,
    for a = 0:maxage;
        LC_ac(a+1,maxage+1-a:maxage+1-a+colnumb-1,page) = LC(a+2,:,page);
        DQ60_MBA_ac(a+1,maxage+1-a:maxage+1-a+colnumb-1,page) = DQ60_MBA(a+2,:,page);
        DQ90_MBA_ac(a+1,maxage+1-a:maxage+1-a+colnumb-1,page) = DQ90_MBA(a+2,:,page);
        DQ120plus_MBA_ac(a+1,maxage+1-a:maxage+1-a+colnumb-1,page) = DQ120plus_MBA(a+2,:,page);
        Runoff_ac(a+1,maxage+1-a:maxage+1-a+colnumb-1,page) = Runoff(a+2,:,page);
        FCPresale_ac(a+1,maxage+1-a:maxage+1-a+colnumb-1,page) = FCPresale(a+2,:,page);
        FCcompleted_ac(a+1,maxage+1-a:maxage+1-a+colnumb-1,page) = FCcompleted(a+2,:,page);
    end
end

% construct default rates
Terminations = DQ60_MBA+DQ90_MBA+DQ120plus_MBA+FCPresale;
Terminations120 = DQ120plus_MBA+FCPresale;
TerminationsFCPre = FCPresale;
Terminations_ac = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
Terminations120_ac = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
TerminationsFCPre_ac = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
for page = 1:sheetnumb,
    for a = 0:maxage;
        Terminations_ac(a+1,maxage+1-a:maxage+1-a+colnumb-1,page) = Terminations(a+2,:,page);
        Terminations120_ac(a+1,maxage+1-a:maxage+1-a+colnumb-1,page) = Terminations120(a+2,:,page);
        TerminationsFCPre_ac(a+1,maxage+1-a:maxage+1-a+colnumb-1,page) = TerminationsFCPre(a+2,:,page);
    end
end
FCcompleted_ac_cumul = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
FCcompleted_ac_reltoLC = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
Runoff_ac_cumul = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
DefTerminations_ac_cumul = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
DefTerminations120_ac_cumul = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
DefTerminationsFCPre_ac_cumul = nan(rownumb-1,colnumb+rownumb-2,sheetnumb);
for ltv = 1:sheetnumb,
    for cohort = 1:colnumb+rownumb-2,
        startage = max(0,maxage+1-cohort);
        endage = min(maxage,colnumb+maxage-cohort);
        FCcompleted_ac_reltoLC(startage+1:endage+1,cohort,ltv) = FCcompleted_ac(startage+1:endage+1,cohort,ltv)./LC_ac(startage+1:endage+1,cohort,ltv);
        active_frac = zeros(maxage+1,1);
        for a = startage:endage;
            if a==startage,
                FCcompleted_ac_cumul(a+1,cohort,ltv) = FCcompleted_ac_reltoLC(a+1,cohort,ltv);
                Runoff_ac_cumul(a+1,cohort,ltv) = Runoff_ac(a+1,cohort,ltv);
                DefTerminations_ac_cumul(a+1,cohort,ltv) = FCcompleted_ac_cumul(a+1,cohort,ltv)+Terminations_ac(a+1,cohort,ltv);
                DefTerminations120_ac_cumul(a+1,cohort,ltv) = FCcompleted_ac_cumul(a+1,cohort,ltv)+Terminations120_ac(a+1,cohort,ltv);
                DefTerminationsFCPre_ac_cumul(a+1,cohort,ltv) = FCcompleted_ac_cumul(a+1,cohort,ltv)+TerminationsFCPre_ac(a+1,cohort,ltv);
            else
                active_frac(a+1) = 1 - FCcompleted_ac_cumul(a+1-1,cohort,ltv) - Runoff_ac_cumul(a+1-1,cohort,ltv);
                FCcompleted_ac_cumul(a+1,cohort,ltv) = FCcompleted_ac_cumul(a+1-1,cohort,ltv)+FCcompleted_ac_reltoLC(a+1,cohort,ltv).*active_frac(a+1);
                Runoff_ac_cumul(a+1,cohort,ltv) = Runoff_ac_cumul(a+1-1,cohort,ltv)+Runoff_ac(a+1,cohort,ltv).*active_frac(a+1);
                DefTerminations_ac_cumul(a+1,cohort,ltv) = FCcompleted_ac_cumul(a+1,cohort,ltv)+Terminations_ac(a+1,cohort,ltv).*active_frac(a+1);
                DefTerminations120_ac_cumul(a+1,cohort,ltv) = FCcompleted_ac_cumul(a+1,cohort,ltv)+Terminations120_ac(a+1,cohort,ltv).*active_frac(a+1);
                DefTerminationsFCPre_ac_cumul(a+1,cohort,ltv) = FCcompleted_ac_cumul(a+1,cohort,ltv)+TerminationsFCPre_ac(a+1,cohort,ltv).*active_frac(a+1);
            end
        end
    end
end

% Save the relevant data
LTV = LTV(2,(time>=2002)&(time<2009),:);
InterestRate = InterestRate(2,(time>=2002)&(time<2009),:);
FICO = FICO(2,(time>=2002)&(time<2009),:);
LoanAmount = LoanAmount(2,(time>=2002)&(time<2009),:);
LC = LC(2,(time>=2002)&(time<2009),:);
Lmax = sum(time>=2002);
DefTerminations_ac_cumul = DefTerminations_ac_cumul(:,maxage+1:end,:);
DefTerminations_ac_cumul = DefTerminations_ac_cumul(1:Lmax,(time>=2002)&(time<2009),:);
DefTerminations120_ac_cumul = DefTerminations120_ac_cumul(:,maxage+1:end,:);
DefTerminations120_ac_cumul = DefTerminations120_ac_cumul(1:Lmax,(time>=2002)&(time<2009),:);
DefTerminationsFCPre_ac_cumul = DefTerminationsFCPre_ac_cumul(:,maxage+1:end,:);
DefTerminationsFCPre_ac_cumul = DefTerminationsFCPre_ac_cumul(1:Lmax,(time>=2002)&(time<2009),:);
FCcompleted_ac_cumul = FCcompleted_ac_cumul(:,maxage+1:end,:);
FCcompleted_ac_cumul = FCcompleted_ac_cumul(1:Lmax,(time>=2002)&(time<2009),:);
save([datapath,'FRMdata'],'DefTerminations_ac_cumul','DefTerminations120_ac_cumul','DefTerminationsFCPre_ac_cumul','FCcompleted_ac_cumul','LTV','InterestRate','FICO','LoanAmount','LC');

% Read in LTV distribution data
rownumb = 22;
colnumb = 138;
sheetnumb = 38;
LTVdistrib = zeros(rownumb,colnumb,sheetnumb);
% Format of LTVdistribution Excel file downloaded from LPS: 
% Variable: Loan Count
% Rows: Age of Loan (Months): row 1: total, then 0 - 20 or more
% Columns: Time Periods (Months): Jan 1999 - June 2010
% Sheets: 
% - Break 1: Original LTV Ratio (Ranges): Total,Less than 75,75,76,...,108,109,110 and more 
% - very first sheet contains an "Index" and very last the "Selections"
% Filter: Mortgage Type is First, and Interest Type is Fixed, and Initial Loan Term EQ 360, and Balloon Code is Not Balloon
disp('load LTV distribution data')
parfor page = 1:sheetnumb,
    disp(['sheet ',num2str(page),' of ',num2str(sheetnumb)]);
    LTVdistrib(:,:,page) = xlsread([datapath,LPSfolder,'LTVdistribution.xls'],page+1);
end
LTVdistrib = LTVdistrib(2,(time>=2002)&(time<2009),:);
save([datapath,'LTVdistribution'],'LTVdistrib');

% Load Regional Composition Data
rownumb = 54;
colnumb = 138;
sheetnumb = 11;
RegSplit0 = zeros(rownumb,colnumb,sheetnumb);
% Format of RegionalSplit0 Excel file downloaded from LPS: 
% Variable: Loan Count
% Rows: State (All values)
% Columns: Time Periods (Months): Jan 1999 - June 2010
% Sheets: 
% - Break 1: Original LTV Ratio (Ranges): Total,1-14,15-24,...,75-84,85-94,95 and more 
% - very first sheet contains an "Index" and very last the "Selections"
% Filter: Mortgage Type is First, and Interest Type is Fixed, and Initial, Loan Term EQ 360, and Balloon Code is Not Balloon, and Loan Age EQ 0
disp('load regional composition data')
parfor page = 1:sheetnumb,
    disp(['sheet ',num2str(page),' of ',num2str(sheetnumb)]);
    RegSplit0(:,:,page) = xlsread([datapath,LPSfolder,'RegionalSplit0.xls'],page+1);
end
RegSplit0 = RegSplit0(:,(time>=2002)&(time<2009),:);
save([datapath,'RegionalSplit'],'RegSplit0');


