% Simulate Structural Model
% for "Mortgage Default during the U.S. Mortgage Crisis"
% by Thomas Schelkle
% Uses: params
%--------------------------------------------------------
%--------------------------------------------------------

function Idefcum = simulmodel(theta,simultime,LTVnumb,resultspath,datapath,I,C,GX,GPH)

params;

% Load Data on Census Division Level House Prices
hp_data = load([datapath,'hpi_cpi_monthly_div.txt']); % US monthly Census division level nominal HPI from Jan1991 to Nov2010
hp_time = (1991:1/12:2010+10/12)';
CPI = hp_data(:,12);
hp_nom = hp_data(:,3:11); % nominal
hp_data = hp_nom./repmat(CPI,1,9); % real
hp_data0 = hp_data(sum(hp_time<min(simultime)),:); % month before start of simulation
hp_data = hp_data((hp_time>=min(simultime))&(hp_time<=max(simultime)),:);
hp_growth_data = (hp_data - [hp_data0;hp_data(1:end-1,:)])./[hp_data0;hp_data(1:end-1,:)];

% Load division level volatility parameters
vol_div = load([datapath,'vol_div.txt']); % division level volatility estimates
kappa_div = vol_div(:,2); % division level quarterly kappa
lambda_div = vol_div(:,3); % division level quarterly lambda
var_rh_ind_div = zeros(9,J);
for di=1:9,
    var_rh_ind_div(di,:) = kappa_div(di)*(4/frequ) + lambda_div(di)*(4/frequ)^2*(2*(1:J)-1); % variance of individual shock depends on time
end
s_rh_ind_div = sqrt(var_rh_ind_div); % division level monthly individual house price volatility

% Load Data on Regional Composition of different Cohorts
S_to_D = load([datapath,'States_to_Divisions.txt']); % division number of states
S_to_D = S_to_D(2:end-2);
load([datapath,'RegionalSplit']); % contains RegSplit0
RegSplit = RegSplit0;
RegSplit(1,:,:) = RegSplit(1,:,:) - RegSplit(end,:,:) - RegSplit(end-1,:,:); %take out Unknown and Other
RegSplit = RegSplit(:,:,LTVnumb+1);
RegSplit = RegSplit(2:end-2,:)./repmat(RegSplit(1,:),size(RegSplit,1)-3,1);

Tsimul = 84; % number of origination dates used in simulation (from Jan2002 to Dec2008)
Lmax = length(simultime); % maximum length of simulation for an individual cohort 
Lind = Lmax:-1:Lmax-Tsimul+1; % simulation length for individual cohort

if opt_var_sf, % load data on time-varying separation and finding rates
    udata = load([datapath,'dur_rate_fm_sm.txt']);
    udata = udata((udata(:,1)>=2002),:); % select data from Jan2002 ...
    udata = udata(1:Lmax,:); % ... until end of simulation
    fm = udata(:,4); % monthly finding rate
    sm = udata(:,5); % monthly separation rate
end
   
% Do a monte carlo simulation for different origination months
ph_agg = zeros(Lmax,Tsimul); % aggregate house price component
ph_div = zeros(Lmax,Tsimul,9); % census division house price component
growth_ph_agg = zeros(Lmax,Tsimul,9); % growth rate of aggregate house price component
approx_growth_ph_agg = zeros(Lmax,Tsimul,9); % approximated growth rate of aggregate hp (the one for which the policy function is used)
g_ph = zeros(Lmax,Tsimul,9); % index of approximated growth rate of aggregate hp (the one for which the policy function is used)

devgridmin = zeros(Lmax,Tsimul);
devgridmax = zeros(Lmax,Tsimul);
above_ph_grid = zeros(Lmax,Tsimul); % number of simulated price paths that fall out of the upper bound of the grid
below_ph_grid =zeros(Lmax,Tsimul); % number of simulated price paths that fall out of the lower bound of the grid
above_x_grid = zeros(Lmax,Tsimul); % number of simulated cash on hand paths that fall out of the upper bound of the grid
below_x_grid =zeros(Lmax,Tsimul); % number of simulated cash on hand paths that fall out of the lower bound of the grid
Numidiv = zeros(9,Tsimul); % regional split in simulation

Idef = zeros(Lmax,Tsimul); % fraction of terminal defaults (unconditional hazard rate)
Isale = zeros(Lmax,Tsimul); % fraction of sales (unconditional hazard rate)
Idefcum = zeros(Lmax,Tsimul); % cumulative terminal defaults
Isalecum = zeros(Lmax,Tsimul); % cumulative sales

Def_Equ = nan(Lmax,Tsimul); % equity of all defaulters

% variables relevant for policy analysis
Avg_NegEqu = zeros(1,Tsimul); % average negative equity of all defaulters in a cohort without bailout 
Avg_Subsidy = zeros(1,Tsimul); % average subsidy per actual defaulter to avoid defaults all together
Tot_Subsidy = zeros(Lmax,Tsimul); % total subsidies to avoid defaults all together

LCmodel = zeros(Lmax,Tsimul); % loan count

Buffervec = zeros(Tsimul,1);

% load policy functions
load([resultspath,'owner_theta',num2str(1000*theta)]);

% set seed for random number generators
rng(seed);
disp(['Simulate model for theta ',num2str(theta)]);

for t_orig = 1:Tsimul, % time of origination index (effectively the cohort index)
        
    % relevant variables for this cohort
    Csimul = zeros(numi,Lmax);
    Xsimul = zeros(numi,Lmax);
    Esimul = zeros(numi,Lmax); % 1=employed,2=unemployed
    Isimul = zeros(numi,Lmax);
    Ihome = ones(numi,Lmax); % 1=homeowner
    
    % calculate number of individuals in each division
    for di = 1:8,
        Numidiv(di,t_orig) = round(numi*sum(RegSplit((S_to_D==di),t_orig)));
    end
    Numidiv(9,t_orig) = numi - sum(Numidiv(1:8,t_orig));
    
    % draw individual house price shocks (new draws for each cohort)
    phindshockmat = zeros(numi,Lind(t_orig));
    for di=1:9,
        if di==1,
            for t=1:Lind(t_orig),
                phindshockmat(1:Numidiv(di,t_orig),t) = s_rh_ind_div(di,t)*randn(Numidiv(di,t_orig),1);
            end
        else
            for t=1:Lind(t_orig),
                phindshockmat(sum(Numidiv(1:di-1,t_orig))+1:sum(Numidiv(1:di,t_orig)),t) = s_rh_ind_div(di,t)*randn(Numidiv(di,t_orig),1);
            end
        end
    end

    % draw income histories
    incuni = rand(numi,Lind(t_orig)); % realizations from standard uniform
    for t = 1:Lind(t_orig),
        if opt_var_sf, % use time-varying separation and finding rates (from data)
            pimatt = [1-sm(t_orig+t-1) sm(t_orig+t-1);fm(t_orig+t-1) 1-fm(t_orig+t-1)];   
        else % use constant separation and finding rates
            pimatt = pimat;
        end
        pilow = [zeros(E,1) cumsum(pimatt(:,1:end-1),2)];
        piup = [cumsum(pimatt(:,1:end-1),2) ones(E,1)];
        
        if t == 1, % all agents have income status einit in period 0
            for enew=1:E,
                Esimul((incuni(:,t)>pilow(einit,enew))&(incuni(:,t)<=piup(einit,enew)),t) = enew;
            end
        else
            for eold=1:E,
                for enew=1:E,
                    Esimul((Esimul(:,t-1)==eold)&(incuni(:,t)>pilow(eold,enew))&(incuni(:,t)<=piup(eold,enew)),t) = enew;
                end   
            end
        end
    end
    
    % hypothetical variables for policy analysis
    if opt_bailout,
        Csimulhyp = zeros(numi,Lmax);
        Xsimulhyp = zeros(numi,Lmax);
        Esimulhyp = Esimul; % same income histories as in actual simulation
        Isimulhyp = zeros(numi,Lmax);
        Ihomehyp = ones(numi,Lmax); % 1=homeowner
    end
    
    % construct house price paths
    ph = zeros(numi,Lind(t_orig)); % house prices faced by this cohort
    ph_ind = zeros(numi,Lind(t_orig)); % individual house price component
    for t=1:Lind(t_orig), % compute individual component
        if t==1,
            ph_ind(:,t) = phindshockmat(:,t);
        else
            ph_ind(:,t) = ph_ind(:,t-1) + phindshockmat(:,t);
        end
    end
    ph_agg_mat = zeros(numi,Lind(t_orig));
    for di=1:9,
        ph_div(1:Lind(t_orig),t_orig,di) = hp_data(t_orig:t_orig+Lind(t_orig)-1,di)/hp_data(t_orig,di)*PH_0;
        growth_ph_agg(1:Lind(t_orig),t_orig,di) = hp_growth_data(t_orig:t_orig+Lind(t_orig)-1,di);
        if di==1,
            ph_agg_mat(1:Numidiv(di,t_orig),:) = repmat(squeeze(ph_div(1:Lind(t_orig),t_orig,di))',Numidiv(di,t_orig),1);
        else
            ph_agg_mat(sum(Numidiv(1:di-1,t_orig))+1:sum(Numidiv(1:di-1,t_orig))+Numidiv(di,t_orig),:) = repmat(squeeze(ph_div(1:Lind(t_orig),t_orig,di))',Numidiv(di,t_orig),1);
        end
        for t=1:Lind(t_orig),
            [approx_growth_ph_agg(t,t_orig,di),g_ph(t,t_orig,di)] = min(abs(Z_agg-growth_ph_agg(t,t_orig,di)));
            approx_growth_ph_agg(t,t_orig,di) = Z_agg(g_ph(t,t_orig,di));
        end
    end
    for t=1:Lind(t_orig),
        ph(:,t) = ph_agg_mat(:,t) .* exp(ph_ind(:,t));
    end
    if Lind(t_orig)>1,
        ph_agg(1:Lind(t_orig),t_orig) = sum(squeeze(ph_div(1:Lind(t_orig),t_orig,:)).*repmat(Numidiv(:,t_orig)',Lind(t_orig),1),2)/numi;
    else
        ph_agg(1:Lind(t_orig),t_orig) = sum(squeeze(ph_div(1:Lind(t_orig),t_orig,:))'.*repmat(Numidiv(:,t_orig)',Lind(t_orig),1),2)/numi;
    end
    
    % calculate desired buffer stock in period 1
    t=1;
    Avec = GX(:,t)-Yinit;
    CX = interpn(GX(:,t),GPH(:,t),squeeze(C(:,:,g_ph(t,t_orig),einit,t)),GX(:,t),PH_0,interpmethsimul);
    distA = Avec - (1+r)*(Avec + Yinit - m(1) - CX);
    [~,indmindistA] = min(abs(distA));
    Buffervec(t_orig) = Avec(indmindistA);
    if Buffervec(t_orig)<-BL(1),
        Buffervec(t_orig) = -BL(1);
    end
    
    if opt_bufferA0,
        % set A0 equal to desired buffer-stock 
        A0 = Buffervec(t_orig);
    else
        % set initial assets to specified value
        A0=0.0027;
    end
    
    for t = 1:Lind(t_orig), % time into the contract for a specific cohort
                
        Ihome1 = (Ihome(:,t)==1); % individuals who still own a house
        if opt_bailout,
            Ihome1hyp = (Ihomehyp(:,t)==1);
        end
        % Check whether house prices remain on the grid
        above_ph_grid(t,t_orig) = sum(ph(Ihome1,t)>GPH(end,t));
        below_ph_grid(t,t_orig) = sum(ph(Ihome1,t)<GPH(1,t));
        % quick fix
        ph((ph(:,t)<GPH(1,t)),t)=GPH(1,t);
        ph((ph(:,t)>GPH(end,t)),t)=GPH(end,t);
        
        if t==1,
            % assume in period 1 everyone is employed
            Xsimul(Ihome1,t) = A0 + Ymat(Esimul(Ihome1,t),t);
            DurEU = ones(numi,Lind(t_orig)); % duration of employment and unemployment spells
            if opt_bailout,
                Xsimulhyp(:,t) = Xsimul(:,t);
            end
        else
            for e=1:E,
                Ihome1ee = ((Ihome(:,t)==1)&(Esimul(:,t-1) == Esimul(:,t))); % same income state again
                DurEU(Ihome1ee,t) = DurEU(Ihome1ee,t-1) + 1;
                Ihome1eu = ((Ihome(:,t)==1)&(Esimul(:,t-1) ~= Esimul(:,t))); % different income state
                DurEU(Ihome1eu,t) = 1;
            end
            
            Xsimul(Ihome1,t) = (1+r)*(Xsimul(Ihome1,t-1)-m(t-1)-Csimul(Ihome1,t-1)) + Ymat(Esimul(Ihome1,t),t);
            
            if opt_bailout,
                Xsimulhyp(Ihome1hyp,t) = (1+r)*(Xsimulhyp(Ihome1hyp,t-1)-m(t-1)-Csimulhyp(Ihome1hyp,t-1)) + Ymat(Esimulhyp(Ihome1hyp,t),t);
            end
            
        end
        
        above_x_grid(t,t_orig) = sum(Xsimul(Ihome1,t)>GX(end,t));
        below_x_grid(t,t_orig) = sum(Xsimul(Ihome1,t)<GX(1,t));
        
        Xsimul(Ihome1,t) = max(Xsimul(Ihome1,t),GX(1,t)); % correct computational imprecision
        
        Vstaysimul = zeros(numi,1);
        Vsellsimul = zeros(numi,1);
        Vdefsimul = zeros(numi,1);
        
        % determine consumption and repayment decision
        for e=1:E,           
            Ihome1e = ((Ihome(:,t)==1)&(Esimul(:,t) == e)); % 
            if opt_simulinterppolicy,
                Isimul(Ihome1e,t) = interpn(GX(:,t),GPH(:,t),squeeze(I(:,:,g_ph(t,t_orig),e,t)),Xsimul(Ihome1e,t),ph(Ihome1e,t),'nearest');  
            else
                Vstaysimul(Ihome1e) = interpn(GX(:,t),GPH(:,t),squeeze(V(:,:,g_ph(t,t_orig),e,t)),Xsimul(Ihome1e,t),ph(Ihome1e,t),interpmethsimul);
                Vsellsimul(Ihome1e) = interp1(GXr(:,t),squeeze(Vr(:,e,t)),Xsimul(Ihome1e,t)+ph(Ihome1e,t)-M(t),interpmethsimul);
                Vdefsimul(Ihome1e) = interp1(GXr(:,t),squeeze(Vr(:,e,t)),Xsimul(Ihome1e,t),interpmethsimul);
                Isimul((Ihome1e)&(Vstaysimul>=Vsellsimul)&(Vstaysimul>=Vdefsimul),t) = 3;
                Isimul((Ihome1e)&(Vsellsimul>Vstaysimul)&(ph(:,t)>=M(t)),t) = 2;
                Isimul((Ihome1e)&(Vdefsimul>Vstaysimul)&(ph(:,t)<M(t)),t) = 1;
            end            
            Csimul(Ihome1e,t) = interpn(GX(:,t),GPH(:,t),squeeze(C(:,:,g_ph(t,t_orig),e,t)),Xsimul(Ihome1e,t),ph(Ihome1e,t),interpmethsimul);
        end 
        
        VioBC3 = ((Ihome(:,t)==1)&(Isimul(:,t)==3)&(Csimul(:,t)+m(t)>Xsimul(:,t)+BL(t)/(1+r)));
            if sum(VioBC3)>0,
                Csimul(VioBC3,t) = Xsimul(VioBC3,t)-m(t)+BL(t)/(1+r);
            end
        
        Isimul1 = ((Ihome(:,t)==1)&(Isimul(:,t) == 1)); % terminal defaulters
        Isimul2 = ((Ihome(:,t)==1)&(Isimul(:,t) == 2)); % sellers
        
        if t<Lind(t_orig),
            Ihome(Isimul1,t+1:end) = 0;
            Ihome(Isimul2,t+1:end) = 0;
        end
        Idef(t,t_orig) = sum(Isimul1)/numi;
        Isale(t,t_orig) = sum(Isimul2)/numi;
        
        if t==1,
            Idefcum(t,t_orig) = Idef(t,t_orig);
            Isalecum(t,t_orig) = Isale(t,t_orig);
        else
            Idefcum(t,t_orig) = Idefcum(t-1,t_orig)+Idef(t,t_orig); 
            Isalecum(t,t_orig) = Isalecum(t-1,t_orig)+Isale(t,t_orig);
        end
        
        if sum(Isimul1)>0,
            Def_Equ(t,t_orig) = mean(ph(Isimul1,t)-M(t)); % equity of all defaulters
        else
            Def_Equ(t,t_orig) = 0; % equity of all defaulters
        end
                
        if opt_bailout,
            Xsimulhyp(Ihome1hyp,t) = max(Xsimulhyp(Ihome1hyp,t),GX(1,t)); % correct computational imprecision
            
            for e = 1:E,
                Ihome1Ehyp = ((Ihomehyp(:,t)==1)&(Esimulhyp(:,t) == e)); % income state this period   
                Isimulhyp(Ihome1Ehyp,t) = interpn(GX(:,t),GPH(:,t),squeeze(I(:,:,g_ph(t,t_orig),e,t)),Xsimulhyp(Ihome1Ehyp,t),ph(Ihome1Ehyp,t),'nearest');
                Isimul1hyp = ((Ihomehyp(:,t)==1)&(Isimulhyp(:,t) == 1)); % hypothetical defaulters
            end
            
            Subsidy = zeros(numi,1);
            for i=1:numi,
                if Isimul1hyp(i) == 1,
                    Ihyp = interpn(GX(:,t),GPH(:,t),squeeze(I(:,:,g_ph(t,t_orig),Esimulhyp(i,t),t)),GX(:,t),ph(i,t),'nearest');
                    Subsidy(i) = GX(sum(Ihyp==1)+1,t) - Xsimulhyp(i,t);
                end
            end
            Xsimulhyp(:,t) = Xsimulhyp(:,t) + Subsidy;
            Tot_Subsidy(t,t_orig) = sum(Subsidy);
            
            for e = 1:E,
                Ihome1Ehyp = ((Ihomehyp(:,t)==1)&(Esimulhyp(:,t) == e)); % income state this period   
                Csimulhyp(Ihome1Ehyp,t) = interpn(GX(:,t),GPH(:,t),squeeze(C(:,:,g_ph(t,t_orig),e,t)),Xsimulhyp(Ihome1Ehyp,t),ph(Ihome1Ehyp,t),interpmethsimul);
            end
                        
            Isimulhyp2 = ((Ihomehyp(:,t)==1)&(Isimulhyp(:,t) == 2)); % sellers
            if t<Lind(t_orig),
                Ihomehyp(Isimulhyp2,t+1:end) = 0;
            end
            
        end
    end
    
    LCmodel(:,t_orig) = sum(Ihome,1)'; % Loan count in model: dimensions Lmax,Tsimul
    
    if opt_bailout,
        Avg_NegEqu(t_orig) = sum( Def_Equ(1:Lind(t_orig),t_orig) *numi .* Idef(1:Lind(t_orig),t_orig) ./ ( (1+r).^(0:Lind(t_orig)-1)' )) / sum(numi*Idef(1:Lind(t_orig),t_orig)); 
        Avg_Subsidy(t_orig) = sum( Tot_Subsidy(1:Lind(t_orig),t_orig) ./ ( (1+r).^(0:Lind(t_orig)-1)' )) / sum(numi*Idef(1:Lind(t_orig),t_orig));
    end
    
end

if (sum(sum(devgridmin<0))>0)||(sum(sum(devgridmax>0))>0),
    disp('HP data falls out of grid range!');
    disp(['Lower limit of GPH needs to be lowered by ',num2str(max(0,-100*min(min(devgridmin)))),' percent!']);
    disp(['Upper limit of GPH needs to be increased by ',num2str(max(0,100*max(max(devgridmax)))),' percent!']);
end

save([resultspath,'simul_theta',num2str(1000*theta)],'ph_agg','Idefcum','Avg_NegEqu','Avg_Subsidy');

disp(['Completed simulation for theta ',num2str(theta)]);

end
