% Computes Policy Functions of Structural Model
% for "Mortgage Default during the U.S. Mortgage Crisis"
% by Thomas Schelkle
% Uses: params
%--------------------------------------------------------
%--------------------------------------------------------

function [I,C,GX,GPH]=solvepolicy(theta,resultspath,datapath,opt_solverentown)

params;

opt_solve_renter = opt_solverentown(1);
opt_solve_owner = opt_solverentown(2);

%--------------------------------------------------------
%--------------------------------------------------------
% Solve Renter Problem

if opt_solve_renter, % solve renter problem
    
disp('Solve RENTER problem');

% increase number of grid points for renter problem
gp_x = gp_x0*gp_x_factor;
gp_s = gp_s0*gp_s_factor;

% form age-dependent grid (use a X and S grids such that borrowing constraint is satisfied)
GX = zeros(gp_x,J);
GS = zeros(gp_s,J);
Xmax = zeros(J,1);
Xmin = zeros(J,1);
for t = 1:J,
    Xmax(t) = GXup_factor*Ymax(t);
    Xmin(t) = Ymin(t);
    if t>1,
        GX(:,t) = makegrid(-BLrent(t-1)+Ymin(t),Xmax(t)+GPHupperbound-M(t),gp_x,curvpara);
    else
        GX(:,t) = makegrid(-BLrent0+Ymin(t),Xmax(t)+GPHupperbound-M(t),gp_x,curvpara);
    end
    GS(:,t) = makegrid(-BLrent(t)/(1+r),Xmax(t)+GPHupperbound-M(t),gp_s,curvpara);
    if BLrent(t)>1, % in case significant borrowing is allowed, place more grid points at relevant parts
        SDense = 1; % have more dense grid points at range [-SDense,SDense] 
        gp_s_dense = 5000;
        GS1 = makegrid(-BLrent(t)/(1+r),-SDense-1e-6,(gp_s-gp_s_dense)/2,curvpara);
        GS2 = makegrid(-SDense,SDense,gp_s_dense,curvpara);
        GS3 = makegrid(SDense+1e-6,Xmax(t)+GPHupperbound-M(t),(gp_s-gp_s_dense)/2,curvpara);
        GS(:,t) = [GS1;GS2;GS3];
    end
end
 
% generate effective rent payment
rentmat = rent*ones(gp_x,J);

% solve for C and V in each period of life as functions of the state variables
% states: x:cash-on-hand, e:employment/income status, t:time 
C = zeros(gp_x,E,J);
V = zeros(gp_x,E,J);

%in last period (J): consume all your wealth 
t = J;
for e=1:E,
    C(:,e,t) = GX(:,t);
    if gama==1,
        V(:,e,t) = log(C(:,e,t));
    else
        V(:,e,t) = exp((1-gama)*log(C(:,e,t))) / (1-gama);
    end
end

for t = J-1:-1:1,        
        %calculate expected value function tomorrow on saving and employment grid
        Vnodes = zeros(gp_s,E); % value tomorrow on saving and future employment grid
        for enext=1:E,
            Xnext = (1+r)*GS(:,t) + Ymat(enext,t+1);
            Xnext((Xnext<GX(1,t+1))) = GX(1,t+1);
            Xnext((Xnext>GX(end,t+1))) = GX(end,t+1);
            Vnodes(:,enext) = interp1(GX(:,t+1),squeeze(V(:,enext,t+1)),Xnext,interpmethsolve);
        end
        EV = zeros(gp_s,E); % expected value tomorrow as a function of saving and conditional on employment state today
        for e=1:E,
            EV(:,e) = Vnodes*pimat(e,:)'; %= pimat(e,1)*Vnodes(:,1) + pimat(e,2)*Vnodes(:,2)  
        end
        
        % calculate optimal consumption for all levels of cash-on-hand and employment status
        for k=1:gp_x, 
            indvecs = (GS(:,t)<GX(k,t)-rentmat(k,t)); % consumption needs to be positive
            GSX = GS(indvecs,t);
            if gama==1,
                UXS = log(GX(k,t)-rentmat(k,t)-GSX); % current flow utility on saving grid
            else
                UXS = exp((1-gama)*log(GX(k,t)-rentmat(k,t)-GSX)) /(1-gama); % current flow utility on saving grid
            end  
            for e=1:E,
                VXS = UXS + betta*EV(indvecs,e);
                [V(k,e,t),maxind] = max(VXS);
                C(k,e,t) = GX(k,t) - rentmat(k,t) - GSX(maxind);
            end
        end
end

% save renter results
Vr = V;
GXr = GX;
save([resultspath,'renter'],'Vr','GXr','rentmat');

disp('RENTER problem solved!!!');

else % load stored renter solution
    load([resultspath,'renter']);
end


%--------------------------------------------------------
%--------------------------------------------------------
% Solve Owner Problem

if opt_solve_owner,

disp(['Solve OWNER problem for theta ',num2str(theta)]);
    
gp_x = gp_x0;
gp_s = gp_s0;
    
GX = zeros(gp_x,J);
GPH = zeros(gp_h,J);
GS = zeros(gp_s,J);
GSmat = nan(gp_s,gp_x);
GCmat = nan(gp_s,gp_x);
maxind = zeros(gp_x,T); % index of maximum element in GS that satisfies BC
Xmax = zeros(J,1);
Xmin = zeros(J,1);
for t = 1:J,
    Xmax(t) = GXup_factor*Ymax(t);
    Xmin(t) = Ymin(t);
    if t>1,
        GX(:,t) = makegrid(-BL(t-1)+Xmin(t),Xmax(t),gp_x,curvpara);
    else
        GX(:,t) = makegrid(-BL0+Xmin(t),Xmax(t),gp_x,curvpara);
    end    
    GS(:,t) = linspace(-BL(t)/(1+r),Xmax(t)-m(t),gp_s);
    if BL(t)>1, % in case significant borrowing is allowed, place more grid points at relevant parts
        SDense = 1;
        GS1 = linspace(-BL(t)/(1+r),-SDense-1e-6,gp_s/2);
        GS2 = linspace(-SDense,Xmax(t)-m(t),gp_s/2);
        GS(:,t) = [GS1,GS2]';
    end
    if M(t)>(1+mu_rh)^t *GPHlowerbound,
        GPH(:,t) = sort([(1+mu_rh)^t*linspace(GPHlowerbound,GPHupperbound,gp_h-1)';M(t)]);
    else
        GPH(:,t) = (1+mu_rh)^t*linspace(GPHlowerbound,GPHupperbound,gp_h);
    end 
end

% solve for C, V and house tenure choices Istay and Isell in each period of life as functions of the state variables
C = zeros(gp_x,gp_h,gp_agg,E,J); % for stayer
Vstay = zeros(gp_x,gp_h,gp_agg,E,J); % for stayer
V = zeros(gp_x,gp_h,gp_agg,E,J); % for owner
I =zeros(gp_x,gp_h,gp_agg,E,J-1);

%in J:
t = J;
for h=1:gp_h,
    for g=1:gp_agg,
        for e=1:E,    
            C(:,h,g,e,t) = GX(:,t)+GPH(h,t); 
            if gama==1,
                V(:,h,g,e,t) = log(C(:,h,g,e,t));
            else
                V(:,h,g,e,t) = exp((1-gama)*log(C(:,h,g,e,t))) / (1-gama);
            end
        end
    end
end

for t = J-1:-1:1,
   
    % discretize shock distribution for the time-dependent variance of the individual shock
    if hp_option == 1, % can combine aggregate and individual
        [nodes_h,weights_h] = qnwnorm(numnodes_h, mu_rh, s_rh(t)^2);
    elseif hp_option == 2, % only individual
        [nodes_h,weights_h] = qnwnorm(numnodes_h, 0, s_rh_ind(t)^2);
    end
        
    % "save" some calculations by placing outside loops: generate GS and GC matrices!
    GSvec = GS(:,t); 
    for k=1:gp_x,
        maxind(k,t) = sum((GSvec<=GX(k,t)-m(t)));
        GSmat(1:maxind(k,t),k) = GSvec(1:maxind(k,t));
        GCmat(1:maxind(k,t),k) = GX(k,t) - m(t) - GSvec(1:maxind(k,t));
    end
    if gama==1,
        UCmat = log(GCmat); % felicity function on GC
    else
        UCmat = exp((1-gama)*log(GCmat))/(1-gama); % felicity function on GC
    end
    
    Xnext = zeros(gp_s,E);
    for enext=1:E,
        Xnext(:,enext) = (1+r)*GSvec + Ymat(enext,t+1); 
        Xnext((Xnext(:,enext)<GX(1,t+1)),enext) = GX(1,t+1);
        Xnext((Xnext(:,enext)>GX(end,t+1)),enext) = GX(end,t+1);
    end
    
    [XX,YY] = meshgrid(GPH(:,t+1),GX(:,t+1)); 
    
    for h=1:gp_h,
        % calculate expected value function for stayer on saving grid conditional on current employment and aggregate hp growth            
        Vnodes = zeros(gp_s,gp_agg,E);
        for g=1:gp_agg, % loop over next period aggregate hp growth rate
            PHnext = GPH(h,t)*exp(Z_agg(g)+nodes_h);
            PHnext((PHnext>GPH(end,t+1))) = GPH(end,t+1); % detect small precision errors and enforce the upper bound
            PHnext((PHnext<GPH(1,t+1))) = GPH(1,t+1); % and enforce lower bound
               
            for enext=1:E,
                [XI,YI] = meshgrid(PHnext,Xnext(:,enext));
                Vnodes(:,g,enext) = interp2(XX,YY,squeeze(V(:,:,g,enext,t+1)),XI,YI,interpmethsolve) * weights_h;
            end
        end
            
        for g=1:gp_agg, % loop over current growth state
            for e=1:E,
                EV = zeros(gp_s,1);
                for enext = 1:E,
                    EV = EV + pimat(e,enext)*squeeze(Vnodes(:,:,enext))*PI_agg(g,:)';
                end
                
                for k=gp_x:-1:1, % calculate optimal consumption on saving grid
                    if maxind(k,t)>0, % can afford a positive consumption level
                        VXS = UCmat(1:maxind(k,t),k) + theta + betta*EV(1:maxind(k,t));
                        [Vstay(k,h,g,e,t),indvec] = max(VXS);
                        C(k,h,g,e,t) = GCmat(indvec,k);
                    else
                        Vstay(k,h,g,e,t) = -inf;
                        C(k,h,g,e,t) = C(k+1,h,g,e,t); % plays only a role in interpolation during simulation
                    end
                end
            end
        end
        V(:,h,:,:,t) = Vstay(:,h,:,:,t);
        I(:,h,:,:,t) = 3;
                        
        % determine tenure choice
        if (GPH(h,t) - M(t) < 0)&&(t<=T), % negative equity during the contract time: consider default
            for e=1:E,
                XDef = GX(:,t);
                VDef = interp1(GXr(:,t),squeeze(Vr(:,e,t)),XDef,interpmethsolve);
                for g=1:gp_agg,
                    def = (Vstay(:,h,g,e,t) < VDef);
                    I(def,h,g,e,t) = 1;
                    V(def,h,g,e,t) = VDef(def);
                end
            end
        else % positive equity: could also sell
            for e=1:E,
                XSale = GX(:,t) + GPH(h,t) - M(t); % current effective X after sale
                XSale((XSale<GXr(1,t))) = GXr(1,t);
                XSale((XSale>GXr(end,t))) = GXr(end,t);
                VSale = interp1(GXr(:,t),squeeze(Vr(:,e,t)),XSale,interpmethsolve);
                for g=1:gp_agg,
                    sell = (Vstay(:,h,g,e,t) < VSale);
                    I(sell,h,g,e,t) = 2;
                    V(sell,h,g,e,t) = VSale(sell);
                end  
            end  
        end
    end        
end

save([resultspath,'owner_theta',num2str(1000*theta)],'I','C','GX','GPH','V');

disp(['OWNER problem for theta',num2str(theta),' solved!!!']);

end

end

function grd = makegrid(x1,x2,nc,curv)

% makes curved grid according to curvature parameter curv
scale=x2-x1;
grd = zeros(nc,1);
grd(1)=x1;
grd(nc)=x2;
for ic=2:nc-1
    grd(ic)=x1+scale*((ic-1)/(nc-1))^curv;
end;

end  
   